@@ -54,14 +54,36 @@ let test (c : connection) =
         REFERENCES postgresql_ocaml_async ON DELETE CASCADE)";
   assert ((fetch_single_result c)#status = Command_ok);
 
+  begin
+    c#send_query
+      ~param_types:Postgresql.[|oid_of_ftype INT8; oid_of_ftype INT8|]
+      ~params:[|"4100100100"; "5100100100"|]
+      "SELECT $1 + $2";
+    let r = fetch_single_result c in
+    assert (r#status = Tuples_ok);
+    assert (r#nfields = 1);
+    assert (r#ntuples = 1);
+    assert (r#getvalue 0 0 = "9200200200");
+  end;
+
   (* Populate using a prepared statement. *)
-  c#send_prepare "test_ins"
-                 "INSERT INTO postgresql_ocaml_async (a, b) VALUES ($1, $2)";
-  assert ((fetch_single_result c)#status = Command_ok);
-  c#send_query_prepared ~params:[|"2"; "two"|] "test_ins";
-  assert ((fetch_single_result c)#status = Command_ok);
-  c#send_query_prepared ~params:[|"3"; "three"|] "test_ins";
-  assert ((fetch_single_result c)#status = Command_ok);
+  let shown_ntuples = 10 in
+  let expected_ntuples = 3 * 100 in
+  for i = 0 to 2 do
+    let stmt = sprintf "test_ins_%d" i in
+    let param_types =
+      Array.sub Postgresql.[|oid_of_ftype INT4; oid_of_ftype TEXT|] 0 i
+    in
+    c#send_prepare stmt ~param_types
+      "INSERT INTO postgresql_ocaml_async (a, b) VALUES ($1, $2)";
+    assert ((fetch_single_result c)#status = Command_ok);
+    for j = 1 to 100 do
+      let c0 = string_of_int (i + 3 * j) in
+      let c1 = sprintf "The number %d." (i + 3 * j) in
+      c#send_query_prepared ~params:[|c0; c1|] stmt;
+      assert ((fetch_single_result c)#status = Command_ok)
+    done
+  done;
 
   (* Prepare a select statement. *)
   c#send_prepare "test_sel" "SELECT * FROM postgresql_ocaml_async";
@@ -80,26 +102,27 @@ let test (c : connection) =
   c#send_query_prepared "test_sel";
   let r = fetch_single_result c in
   assert (r#status = Tuples_ok);
-  assert (r#ntuples = 2);
+  assert (r#ntuples = expected_ntuples);
   assert (r#nfields = 3);
-  for i = 0 to r#ntuples - 1 do
-    Printf.printf "%s %s %s\n"
-                  (r#getvalue i 0) (r#getvalue i 1) (r#getvalue i 2)
+  for i = 0 to min r#ntuples shown_ntuples - 1 do
+    printf "%s, %s, %s\n" (r#getvalue i 0) (r#getvalue i 1) (r#getvalue i 2)
   done;
+  printf "[...]\n";
 
   (* Run it in single-row mode. *)
   c#send_query_prepared "test_sel";
   c#set_single_row_mode;
-  for i = 0 to 2 do
+  for i = 0 to expected_ntuples do
     match fetch_result c with
     | None -> assert false
-    | Some r when i < 2 ->
+    | Some r when i < expected_ntuples ->
       assert (r#status = Single_tuple);
-      Printf.printf "%s %s %s\n"
-                    (r#getvalue 0 0) (r#getvalue 0 1) (r#getvalue 0 2)
+      if i < shown_ntuples then
+        printf "%s, %s, %s\n" (r#getvalue 0 0) (r#getvalue 0 1) (r#getvalue 0 2)
     | Some r ->
       assert (r#status = Tuples_ok)
   done;
+  printf "[...]\n";
   assert (fetch_result c = None);
 
   (* Drop the main table. *)
